#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <sly_@ifrance.com>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <sly_@ifrance.com>
##############################################################################
##############################################################################
## Define Gwine::Config class
#############################################################################
package Gwine::Config;
use strict;
use warnings;

BEGIN {
    use vars qw( @ISA %fields $AUTOLOAD $permitted_fields );
    $permitted_fields = '_permitted_fields';
}

%fields = (
	   # Informations to save on disk
	   # Last opened file
	   opened_file => undef,
	   # say if opened_file must be used
	   save_opened_file => 0,
	   # Size of main windown (geometry = ${width}x$height)
	   geometry => undef,
	   # Position of divider between panes
	   paned_pos => undef,
	   # say if geometry and hpaned_pos must be used
	   save_geometry => 0, 
	   # Use confirmation popups (or not)
	   conf_popups => 1,
	   # Columns that must be visible in wine list
	   wine_visible => [1, 1, 1, 1, 1, 1, 1, 1, 0],
	   # Columns that must be visible in prod list
	   prod_visible => [1, 1, 1, 1, 1, 0],
	   # HOME directory
	   home => undef,
	   # save last clicked between lists or tree
	   last_clicked => undef,
	   # save locale decimal point
#	   locale_dp => undef,
	   # save icon path
	   icon_path => undef,
	   );

# List of fields to be saved on disk by save method
# These fields may be set by restore method
my @to_save = qw( opened_file save_opened_file paned_pos geometry 
                  save_geometry conf_popups wine_visible 
                  prod_visible );



##############################################################################
## Create a new Gwine::Config object
##############################################################################
sub new {
  my $this = shift;
  my $class = ref($this) || $this;

  my $self = {
      $permitted_fields => \%fields, %fields
  };
  bless $self, $class;
  return $self;
}


##############################################################################
## AUTOLOAD
##############################################################################
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
        or die "$self is not an object so we cannot '$AUTOLOAD'\n",
            "We were called from ".join(", ", caller)."\n\n";
    my $name = $AUTOLOAD;
    $name =~ s/.*://;       # strip fully-qualified portion

    if (exists $self->{$permitted_fields}{$name} ) {
        # This allows dynamic data methods - see hash fields above
        # eg $class->UI('new_value');
        # or $current_value = $class->UI;
        if (@_) {
            return $self->{$name} = shift;
        } else {
            return $self->{$name};
        }

    } else {
        die "Can't access method `$name' in class $type\n".
            "We were called from ".join(", ", caller)."\n\n";
    }
}


##############################################################################
## get_opened_file. Take care of save_opened_file
##############################################################################
sub get_opened_file {
    my $self = shift;
    
    if ($self->{save_opened_file}) { return $self->{opened_file} }
    else { return }
}

##############################################################################
## get_geometry. Take care of save_geometry
##############################################################################
sub get_geometry {
    my $self = shift;
    
    if ($self->{save_geometry}) { return $self->{geometry} }
    else { return }
}

##############################################################################
## Restore Config from $HOME/.gwinerc
##############################################################################
sub restore {
    my $self = shift;

    my $file = $self->{home} . '/.gwinerc';

    # Restore from local settings
    use POSIX qw(locale_h);
    my ($locale_dp) = @{localeconv()}{'decimal_point'};
    no POSIX;
    $self->{locale_dp} = $locale_dp;

    # Restore from file
    if (open(GWINERC, "<$file")) {
	while(<GWINERC>) {
	    foreach my $item (@to_save) {
		if (/^$item=(.*)$/) { 
		    my $t = $1; 
		    if ($t =~ /,/) {
			my @a = split /,/, $t;
			$self->{$item} = \@a;
		    }
		    else { $self->{$item} = $t }
		}
	    }
	}
	close GWINERC;
    }
    else { return 0 }
    return 1;
}


##############################################################################
## Save Config to $HOME/.gwinerc
##############################################################################
sub save  {
    my $self = shift;
    
    my $file = $self->{home} . '/.gwinerc';
    if (open(GWINERC, ">$file")) {
	foreach my $item (@to_save) {
	    if ($self->{$item} =~ /ARRAY/) {
		print GWINERC "$item=", 
		    join ',', @{$self->{$item}}, "\n";
	    }
	    else {
		print GWINERC "$item=", $self->{$item}, "\n"
	    }
	}
	close GWINERC
    }
    else { return 0 }

    return 1
}


sub DESTROY {
}


# Now, this is doing by L10N methods
#############################################################################
## Get a float and set its decimal point to locale dp
#############################################################################
#sub set_to_locale_dp {
#    my $self = shift;
#    my $float = shift;
#
#    use locale;
#    my $string = sprintf("%.2f", $float + 0.0);
#    no locale;
#    return $string;
#}
#
#
#############################################################################
## From a "float" string, set decimal point to C dp (".")
#############################################################################
#sub set_to_c_dp {
#    my $self = shift;
#    my $float = shift;
#
#    my $locale_dp = $self->{locale_dp};
#    $float =~ s|$locale_dp|\.|;
#    return $float;
#}



1;

__END__
