#!/usr/bin/perl
##############################################################################
# Copyright (c) 2003
# Author Sylvain Daubert <sly_@ifrance.com>
#
# This program is free software; you can redistribute it and/or modify it 
# under the terms of the GNU General Public License as published by the Free 
# Software Foundation; either version 2 of the License, or (at your option) 
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 675 Mass 
# Ave, Cambridge, MA 02139, USA.
# Sylvain Daubert <sly_@ifrance.com>
#############################################################################
use strict;
use warnings;

####################################################################
## Package Gwine::L10N
## Base class for all L10N classes
####################################################################

package Gwine::L10N;
BEGIN {
    use Locale::Maketext;
    use base qw( Locale::Maketext );
}

#use Number::Format;
use DateTime;


#############################################################################
## called by Locale::Maketext->get_handle
sub init {
    my $self = shift;

    # Create a DateTime object
    use POSIX qw( locale_h );
    my $locale = setlocale(LC_ALL);
    no POSIX;
    # DateTime::Locale uses aa_AA_variant format
    # But, on linux, locales use aa_AA[.charset][@variant]
    # First, suppress charset
    print "Locale: $locale\n";
    $locale =~ s/\.[\w\d-]+//;
    # Then, change @euro variant to PREEURO
    $locale =~ s/\@euro/_PREEURO/;
    $self->{datetime} = DateTime->new( year => 2003, locale => $locale );
    # Create a number formatter. 
    $self->{decimal_digits} = 2;
    $self->{decimal_fill} = 1;
    $self->{decimal_point} = '.';
#    if (@_) {
#	$self->{number_f} = Number::Format->new(@_);
#    }
#    else {
#	$self->{number_f} = Number::Format->new(-decimal_digits => 2,
#						-decimal_fill   => 1)
#    }
}


#############################################################################
## Overriding Locale::Maketext->numf.
## Format a number according to locale
## If $number is already formatting according to locale, just return it
sub numf {
    my $self = shift;
    my $number = shift;
    
    my $dp = $self->{decimal_point};
    return $number if $number =~ /$dp/;

    my $format = '%.' . ($self->{decimal_fill} ? $self->{decimal_digits} : 0) 
	. 'f';
    $number = sprintf $format, $number;
    $number =~ s/\./$dp/;
    return $number;
}


#############################################################################
# Method to unformat a number.
sub numunf {
    my $self = shift;
    my $number = shift;
    
    my $dp = $self->{decimal_point};
    $number =~ s/$dp/\./;
    return $number;
}


#############################################################################
# Method to format a date according to locale.
sub to_locale_format {
    my $self = shift;
    return $self->{datetime}->strftime('%m-%d-%Y');
}


#############################################################################
# Method to format a date according to locale from save format (yyyy-mm-dd).
sub save2date {
    my $self = shift;
    my $save = shift;
    
    $save =~ m/(\d+)-(\d+)-(\d+)/;
    my ($year, $month, $day) = ($1, $2, $3);
    $self->{datetime}->set(year => $year, month => $month, day => $day);
    return $self->to_locale_format;
}


#############################################################################
# Method to unformat a date from locale to save format (yyyy-mm-dd).
sub date2save {
    my $self = shift;
    my $date = shift;

    my @format = split //, $self->{datetime}->locale->date_parts_order;
    foreach my $f (@format) {
	if ($f eq 'y') {
	    $date =~ s/^(\d+)\D?//;
	    my $year = $1;
	    $self->{datetime}->set(year => $year);
	    next
	}
	if ($f eq 'm') {
	    $date =~ s/^(\d+)\D?//;
	    my $month = $1;
	    $self->{datetime}->set(month => $month);
	    next
	}
	if ($f eq 'd') {
	    $date =~ s/^(\d+)\D?//;
	    my $day = $1;
	    $self->{datetime}->set(day => $day);
	    next
	}
    }
    return $self->{datetime}->ymd('-')
}



1;


__END__
